@@ -25,7 +25,6 @@ from traitlets import (Any, Bool, Dict, List, Unicode, TraitError, Integer,
 
 from jupyter_server.utils import to_os_path, ensure_async
 from jupyter_server._tz import utcnow, isoformat
-from ipython_genutils.py3compat import getcwd
 
 from jupyter_server.prometheus.metrics import KERNEL_CURRENTLY_RUNNING_TOTAL
 
@@ -58,7 +57,7 @@ class MappingKernelManager(MultiKernelManager):
         try:
             return self.parent.root_dir
         except AttributeError:
-            return getcwd()
+            return os.getcwd()
 
     @validate('root_dir')
     def _update_root_dir(self, proposal):
@@ -180,9 +179,11 @@ class MappingKernelManager(MultiKernelManager):
             The name identifying which kernel spec to launch. This is ignored if
             an existing kernel is returned, but it may be checked in the future.
         """
-        if kernel_id is None:
+        if kernel_id is None or kernel_id not in self:
             if path is not None:
                 kwargs['cwd'] = self.cwd_for_path(path)
+            if kernel_id is not None:
+                kwargs['kernel_id'] = kernel_id
             kernel_id = await ensure_async(self.pinned_superclass.start_kernel(self, **kwargs))
             self._kernel_connections[kernel_id] = 0
             self._kernel_ports[kernel_id] = self._kernels[kernel_id].ports
@@ -202,7 +203,6 @@ class MappingKernelManager(MultiKernelManager):
             ).inc()
 
         else:
-            self._check_kernel_id(kernel_id)
             self.log.info("Using existing kernel: %s" % kernel_id)
 
         # Initialize culling if not already
@@ -252,11 +252,11 @@ class MappingKernelManager(MultiKernelManager):
         ----------
         kernel_id : str
             The id of the kernel to stop buffering.
-        session_key: str
+        session_key : str
             The session_key, if any, that should get the buffer.
             If the session_key matches the current buffered session_key,
             the buffer will be returned.
-        channels: dict({'channel': ZMQStream})
+        channels : dict({'channel': ZMQStream})
             The zmq channels whose messages should be buffered.
         """
 
@@ -291,7 +291,7 @@ class MappingKernelManager(MultiKernelManager):
         ----------
         kernel_id : str
             The id of the kernel to stop buffering.
-        session_key: str, optional
+        session_key : str, optional
             The session_key, if any, that should get the buffer.
             If the session_key matches the current buffered session_key,
             the buffer will be returned.
